@@ -6,8 +6,8 @@ Area Detector Support
 
    ~AD_EpicsFileNameHDF5Plugin
    ~AD_EpicsFileNameJPEGPlugin
-   ~AD_EpicsFileNameTIFFPlugin
    ~AD_EpicsFileNameMixin
+   ~AD_EpicsFileNameTIFFPlugin
    ~AD_EpicsHdf5FileName
    ~AD_EpicsHDF5IterativeWriter
    ~AD_EpicsJPEGFileName
@@ -19,24 +19,34 @@ Area Detector Support
    ~AD_prime_plugin
    ~AD_prime_plugin2
    ~AD_setup_FrameType
+   ~CamMixin_V3_1_1
+   ~CamMixin_V34
+   ~SingleTrigger_V34
 """
 
+import datetime
+import itertools
+import logging
+import pathlib
+import time
+import warnings
 from collections import OrderedDict
+
+import epics
+import numpy as np
+from ophyd import EpicsSignal
+from ophyd import EpicsSignalRO
+from ophyd import EpicsSignalWithRBV
+from ophyd.areadetector import ADComponent
+from ophyd.areadetector import CamBase
+from ophyd.areadetector import SingleTrigger
 from ophyd.areadetector.filestore_mixins import FileStoreBase
 from ophyd.areadetector.filestore_mixins import FileStoreIterativeWrite
 from ophyd.areadetector.filestore_mixins import FileStorePluginBase
 from ophyd.areadetector.plugins import HDF5Plugin_V34 as HDF5Plugin
 from ophyd.areadetector.plugins import JPEGPlugin_V34 as JPEGPlugin
 from ophyd.areadetector.plugins import TIFFPlugin_V34 as TIFFPlugin
-from ophyd.utils import set_and_wait
-import datetime
-import epics
-import itertools
-import logging
-import numpy as np
-import pathlib
-import time
-import warnings
+from packaging import version
 
 logger = logging.getLogger(__name__)
 
@@ -231,13 +241,13 @@ def AD_prime_plugin2(plugin):
 
     for sig, val in sigs.items():
         time.sleep(0.1)  # abundance of caution
-        set_and_wait(sig, val)
+        sig.set(val).wait()
 
     time.sleep(2)  # wait for acquisition
 
     for sig, val in reversed(list(original_vals.items())):
         time.sleep(0.1)
-        set_and_wait(sig, val)
+        sig.set(val).wait()
 
 
 def AD_full_file_name_local(plugin):
@@ -265,23 +275,26 @@ def AD_full_file_name_local(plugin):
         return ffname
 
     # identify the common last parts of the file directories
-    rparts = pathlib.Path(plugin.read_path_template).parts
-    wparts = pathlib.Path(plugin.write_path_template).parts
+    read_parts = pathlib.Path(plugin.read_path_template).parts
+    write_parts = pathlib.Path(plugin.write_path_template).parts
     icommon = 0
-    for i in range(min(len(rparts), len(wparts))):
+    for i in range(min(len(read_parts), len(write_parts))):
         i1 = -i - 1
-        if rparts[i1:] != wparts[i1:]:
+        if read_parts[i1:] != write_parts[i1:]:
             icommon = i
             break
 
+    # fmt: off
     if icommon == 0:
         raise ValueError(
-            "No common part to file paths. " "Cannot convert to local file path."
+            "No common part to file paths. "
+            "Cannot convert to local file path."
         )
+    # fmt: on
 
     # fmt: off
-    local_root = pathlib.Path().joinpath(*rparts[:-icommon])
-    common_parts = ffname.parts[len(wparts[:-icommon]):]
+    local_root = pathlib.Path().joinpath(*read_parts[:-icommon])
+    common_parts = ffname.parts[len(write_parts[:-icommon]):]
     local_ffname = local_root.joinpath(*common_parts)
     # fmt: on
 
@@ -341,16 +354,16 @@ class AD_EpicsFileNameMixin(FileStorePluginBase):
     def _remove_caller_stage_sigs(self):
         """Caller is responsible for setting these stage_sigs."""
         caller_sets_these = """
-        array_counter
-        auto_increment
-        auto_save
-        compression (only HDF)
-        create_directory
-        file_name
-        file_number
-        file_path
-        file_template
-        num_capture
+            array_counter
+            auto_increment
+            auto_save
+            compression (only HDF)
+            create_directory
+            file_name
+            file_number
+            file_path
+            file_template
+            num_capture
         """.split()
         for key in caller_sets_these:
             if key in self.stage_sigs:
@@ -390,16 +403,19 @@ class AD_EpicsFileNameMixin(FileStorePluginBase):
         Set EPICS items before device is staged, then copy EPICS
         naming template (and other items) to ophyd after staging.
         """
+        if "capture" in self.stage_sigs:
+            self.stage_sigs.move_to_end("capture", last=True)
+
         # Get the file name and paths from EPICS.
         filename, read_path, write_path = self.make_filename()
 
         # Ensure we do not have an old file open.
-        set_and_wait(self.capture, 0)
+        self.capture.set(0).wait()
 
         # Set these before capture is turned on.
         # They will not be reset on 'unstage' anyway.
-        set_and_wait(self.file_path, write_path)
-        set_and_wait(self.file_name, filename)
+        self.file_path.set(write_path).wait()
+        self.file_name.set(filename).wait()
 
         # Get file number now, it is incremented during stage().
         file_number = self.file_number.get()
@@ -447,7 +463,7 @@ class AD_EpicsHdf5FileName(AD_EpicsFileNameMixin):
         )
         self._remove_caller_stage_sigs()
         # "capture" must always come last
-        self.stage_sigs["capture"] = self.stage_sigs.pop("capture")
+        self.stage_sigs.move_to_end("capture", last=True)
 
 
 class AD_EpicsHDF5IterativeWriter(AD_EpicsHdf5FileName, FileStoreIterativeWrite):
@@ -472,11 +488,12 @@ class AD_EpicsFileNameHDF5Plugin(HDF5Plugin, AD_EpicsHDF5IterativeWriter):
 
     EXAMPLE::
 
+        from apstools.devices import CamMixin_V34
+        from apstools.devices import SingleTrigger_V34
         from apstools.devices.area_detector_support import AD_EpicsFileNameHDF5Plugin
         from ophyd import EpicsSignalWithRBV
         from ophyd.areadetector import ADComponent
         from ophyd.areadetector import DetectorBase
-        from ophyd.areadetector import SingleTrigger
         from ophyd.areadetector.plugins import ImagePlugin_V34 as ImagePlugin
         from ophyd.areadetector.plugins import PvaPlugin_V34 as PvaPlugin
         from ophyd.areadetector import SimDetectorCam
@@ -493,16 +510,13 @@ class AD_EpicsFileNameHDF5Plugin(HDF5Plugin, AD_EpicsHDF5IterativeWriter):
         WRITE_PATH_TEMPLATE = f"{AD_IOC_MOUNT_PATH / IMAGE_DIR}/"
         READ_PATH_TEMPLATE = f"{BLUESKY_MOUNT_PATH / IMAGE_DIR}/"
 
+        class SimDetectorCam_V34(CamMixin_V34, SimDetectorCam):
+            '''Revise SimDetectorCam for ADCore revisions.'''
 
-        class MyFixedCam(SimDetectorCam):
-            pool_max_buffers = None
-            offset = ADComponent(EpicsSignalWithRBV, "Offset")
-
-
-        class MySimDetector(SingleTrigger, DetectorBase):
+        class SimDetector_V34(SingleTrigger_V34, DetectorBase):
             '''ADSimDetector'''
 
-            cam = ADComponent(MyFixedCam, "cam1:")
+            cam = ADComponent(SimDetectorCam_V34, "cam1:")
             image = ADComponent(ImagePlugin, "image1:")
             hdf1 = ADComponent(
                 AD_EpicsFileNameHDF5Plugin,
@@ -536,7 +550,7 @@ class AD_EpicsJPEGFileName(AD_EpicsFileNameMixin):
         )
         self._remove_caller_stage_sigs()
         # "capture" must always come last
-        self.stage_sigs["capture"] = self.stage_sigs.pop("capture")
+        self.stage_sigs.move_to_end("capture", last=True)
 
 
 class AD_EpicsJPEGIterativeWriter(AD_EpicsJPEGFileName, FileStoreIterativeWrite):
@@ -561,11 +575,12 @@ class AD_EpicsFileNameJPEGPlugin(JPEGPlugin, AD_EpicsJPEGIterativeWriter):
 
     EXAMPLE::
 
+        from apstools.devices import CamMixin_V34
+        from apstools.devices import SingleTrigger_V34
         from apstools.devices.area_detector_support import AD_EpicsFileNameJPEGPlugin
         from ophyd import EpicsSignalWithRBV
         from ophyd.areadetector import ADComponent
         from ophyd.areadetector import DetectorBase
-        from ophyd.areadetector import SingleTrigger
         from ophyd.areadetector.plugins import ImagePlugin_V34 as ImagePlugin
         from ophyd.areadetector.plugins import PvaPlugin_V34 as PvaPlugin
         from ophyd.areadetector import SimDetectorCam
@@ -583,15 +598,14 @@ class AD_EpicsFileNameJPEGPlugin(JPEGPlugin, AD_EpicsJPEGIterativeWriter):
         READ_PATH_TEMPLATE = f"{BLUESKY_MOUNT_PATH / IMAGE_DIR}/"
 
 
-        class MyFixedCam(SimDetectorCam):
-            pool_max_buffers = None
-            offset = ADComponent(EpicsSignalWithRBV, "Offset")
+        class SimDetectorCam_V34(CamMixin_V34, SimDetectorCam):
+            '''Revise SimDetectorCam for ADCore revisions.'''
 
 
-        class MySimDetector(SingleTrigger, DetectorBase):
+        class SimDetector_V34(SingleTrigger_V34, DetectorBase):
             '''ADSimDetector'''
 
-            cam = ADComponent(MyFixedCam, "cam1:")
+            cam = ADComponent(SimDetectorCam_V34, "cam1:")
             image = ADComponent(ImagePlugin, "image1:")
             jpeg1 = ADComponent(
                 AD_EpicsFileNameHDF5Plugin,
@@ -625,7 +639,7 @@ class AD_EpicsTIFFFileName(AD_EpicsFileNameMixin):
         )
         self._remove_caller_stage_sigs()
         # "capture" must always come last
-        self.stage_sigs["capture"] = self.stage_sigs.pop("capture")
+        self.stage_sigs.move_to_end("capture", last=True)
 
 
 class AD_EpicsTIFFIterativeWriter(AD_EpicsTIFFFileName, FileStoreIterativeWrite):
@@ -650,11 +664,12 @@ class AD_EpicsFileNameTIFFPlugin(TIFFPlugin, AD_EpicsTIFFIterativeWriter):
 
     EXAMPLE::
 
+        from apstools.devices import CamMixin_V34
+        from apstools.devices import SingleTrigger_V34
         from apstools.devices.area_detector_support import AD_EpicsFileNameTIFFPlugin
         from ophyd import EpicsSignalWithRBV
         from ophyd.areadetector import ADComponent
         from ophyd.areadetector import DetectorBase
-        from ophyd.areadetector import SingleTrigger
         from ophyd.areadetector.plugins import ImagePlugin_V34 as ImagePlugin
         from ophyd.areadetector.plugins import PvaPlugin_V34 as PvaPlugin
         from ophyd.areadetector import SimDetectorCam
@@ -672,15 +687,14 @@ class AD_EpicsFileNameTIFFPlugin(TIFFPlugin, AD_EpicsTIFFIterativeWriter):
         READ_PATH_TEMPLATE = f"{BLUESKY_MOUNT_PATH / IMAGE_DIR}/"
 
 
-        class MyFixedCam(SimDetectorCam):
-            pool_max_buffers = None
-            offset = ADComponent(EpicsSignalWithRBV, "Offset")
+        class SimDetectorCam_V34(CamMixin_V34, SimDetectorCam):
+            '''Revise SimDetectorCam for ADCore revisions.'''
 
 
-        class MySimDetector(SingleTrigger, DetectorBase):
+        class SimDetector_V34(SingleTrigger_V34, DetectorBase):
             '''ADSimDetector'''
 
-            cam = ADComponent(MyFixedCam, "cam1:")
+            cam = ADComponent(SimDetectorCam_V34, "cam1:")
             image = ADComponent(ImagePlugin, "image1:")
             tiff1 = ADComponent(
                 AD_EpicsFileNameTIFFPlugin,
@@ -696,12 +710,72 @@ class AD_EpicsFileNameTIFFPlugin(TIFFPlugin, AD_EpicsTIFFIterativeWriter):
     pass
 
 
+class CamMixin_V3_1_1(CamBase):
+    """
+    Update cam support to AD release 3.1.1.
+
+    (new in release 1.6.3)
+    """
+
+    _cam_release = "3.1.1"
+    pool_max_buffers = None
+    acquire_busy = ADComponent(EpicsSignalRO, "AcquireBusy")
+    offset = ADComponent(EpicsSignalWithRBV, "Offset")
+    wait_for_plugins = ADComponent(EpicsSignal, "WaitForPlugins")
+
+    @property
+    def is_busy(self):
+        signal = self.acquire_busy
+        return signal.get() in (1, signal.enum_strs[1])
+
+
+class CamMixin_V34(CamMixin_V3_1_1):
+    """
+    Update cam support to AD release 3.1.1.
+
+    (new in release 1.6.3)
+    """
+
+    _cam_release = "3.4"
+
+
+class SingleTrigger_V34(SingleTrigger):
+    """
+    Variation of ophyd's SingleTrigger mixin supporting AcquireBusy.
+
+    (new in release 1.6.3)
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        has_v34_cam_features = (
+            hasattr(self, "cam")
+            and hasattr(self.cam, "_cam_release")
+            and version.parse(self.cam._cam_release) >= version.parse("3.4")
+        )
+        if has_v34_cam_features:
+            self._acquisition_busy_signal = self.cam.acquire_busy
+        else:
+            # backwards compatibility
+            self._acquisition_busy_signal = self._acquisition_signal
+
+    def stage(self):
+        """Prepare device settings before data acquisition."""
+        self._acquisition_busy_signal.subscribe(self._acquire_changed)
+        super(SingleTrigger, self).stage()  # from grandparent
+
+    def unstage(self):
+        """Restore device settings after data acquisition."""
+        super(SingleTrigger, self).unstage()  # from grandparent
+        self._acquisition_busy_signal.clear_sub(self._acquire_changed)
+
+
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 # -----------------------------------------------------------------------------
