@@ -9,6 +9,7 @@ Mixin to add EPICS .DESC field
 
 from ophyd import Component
 from ophyd import EpicsSignal
+
 from .mixin_base import DeviceMixinBase
 
 
@@ -24,6 +25,7 @@ class EpicsDescriptionMixin(DeviceMixinBase):
         from apstools.devices import EpicsDescriptionMixin
 
         class MyEpicsMotor(EpicsDescriptionMixin, EpicsMotor): pass
+
         m1 = MyEpicsMotor('xxx:m1', name='m1')
         print(m1.desc.get())
 
@@ -31,8 +33,10 @@ class EpicsDescriptionMixin(DeviceMixinBase):
 
         class TunableSynAxis(AxisTunerMixin, SynAxis):
             '''synthetic axis that can be tuned'''
+
         class TunableEpicsMotor(AxisTunerMixin, EpicsMotor):
             '''EpicsMotor that can be tuned'''
+
         class EpicsMotorWithDescription(EpicsDescriptionMixin, EpicsMotor):
             '''EpicsMotor with description field'''
 
@@ -41,7 +45,8 @@ class EpicsDescriptionMixin(DeviceMixinBase):
             EpicsMotorLimitsMixin,
             EpicsMotorDialMixin,
             EpicsMotorRawMixin,
-            EpicsMotor):
+            EpicsMotor
+        ):
             '''
             EpicsMotor with more fields
 
@@ -54,13 +59,12 @@ class EpicsDescriptionMixin(DeviceMixinBase):
 
     desc = Component(EpicsSignal, ".DESC")
 
-
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 # -----------------------------------------------------------------------------
