@@ -1,39 +1,34 @@
-""" FIXME:
-(bluesky_2022_3) prjemian@zap:~/.../BCDA-APS/apstools$ pytest -vvv --lf ./apstools/devices/tests/test_area_detector_support.py
-...
-apstools/devices/area_detector_support.py:418: ValueError
-============================================================== short test summary info ===============================================================
-FAILED apstools/devices/tests/test_area_detector_support.py::test_stage[tiff1-AD_EpicsFileNameTIFFPlugin] - OSError: Path '/' does not exist on IOC.
-FAILED apstools/devices/tests/test_area_detector_support.py::test_acquire[jpeg1-AD_EpicsFileNameJPEGPlugin] - ValueError: incomplete format
-FAILED apstools/devices/tests/test_area_detector_support.py::test_acquire[tiff1-AD_EpicsFileNameTIFFPlugin] - AttributeError: 'NoneType' object has...
-FAILED apstools/devices/tests/test_area_detector_support.py::test_full_file_name_local - ValueError: incomplete format
-FAILED apstools/devices/tests/test_area_detector_support.py::test_file_numbering - ValueError: incomplete format
-========================================================== 5 failed, 4 deselected in 7.88s ===========================================================
-============================================================== short test summary info ===============================================================
-FAILED apstools/devices/tests/test_area_detector_support.py::test_acquire[jpeg1-AD_EpicsFileNameJPEGPlugin] - AttributeError: 'NoneType' object has...
-FAILED apstools/devices/tests/test_area_detector_support.py::test_acquire[tiff1-AD_EpicsFileNameTIFFPlugin] - AttributeError: 'NoneType' object has...
-FAILED apstools/devices/tests/test_area_detector_support.py::test_file_numbering - AssertionError: assert False > 0
-===================================================== 3 failed, 2 passed, 4 deselected in 19.05s =====================================================
+"""
+Test code where users control the names of the image files using EPICS.
 """
 
-from ...devices import AD_EpicsFileNameHDF5Plugin
-from ...devices import AD_EpicsFileNameJPEGPlugin
-from ...devices import AD_EpicsFileNameTIFFPlugin
-from ...devices import AD_full_file_name_local
-from ...devices import AD_prime_plugin2
-from ophyd import EpicsSignalWithRBV
-from ophyd.areadetector import ADComponent
-from ophyd.areadetector import DetectorBase
-from ophyd.areadetector import SingleTrigger
-from ophyd.areadetector.plugins import ImagePlugin_V34 as ImagePlugin
-from ophyd.areadetector.plugins import PvaPlugin_V34 as PvaPlugin
-from ophyd.areadetector import SimDetectorCam
-from ophyd.signal import EpicsSignalBase
 import datetime
 import pathlib
-import pytest
+import random
+import tempfile
 import time
 
+import bluesky
+import bluesky.plan_stubs as bps
+import bluesky.plans as bp
+import pytest
+from ophyd.areadetector import ADComponent
+from ophyd.areadetector import DetectorBase
+from ophyd.areadetector import SimDetectorCam
+from ophyd.areadetector.plugins import ImagePlugin_V34 as ImagePlugin
+from ophyd.areadetector.plugins import PvaPlugin_V34 as PvaPlugin
+from ophyd.signal import EpicsSignalBase
+
+from ...tests import MASTER_TIMEOUT
+from .. import AD_EpicsFileNameHDF5Plugin
+from .. import AD_EpicsFileNameJPEGPlugin
+from .. import AD_EpicsFileNameMixin
+from .. import AD_EpicsFileNameTIFFPlugin
+from .. import AD_full_file_name_local
+from .. import AD_plugin_primed
+from .. import AD_prime_plugin2
+from .. import CamMixin_V34 as CamMixin
+from .. import SingleTrigger_V34 as SingleTrigger
 
 IOC = "ad:"
 IMAGE_DIR = "adsimdet/%Y/%m/%d"
@@ -49,23 +44,22 @@ READ_PATH_TEMPLATE = f"{BLUESKY_MOUNT_PATH / IMAGE_DIR}/"
 try:
     EpicsSignalBase.set_defaults(
         auto_monitor=True,
-        timeout=60,
-        write_timeout=60,
-        connection_timeout=60,
+        timeout=MASTER_TIMEOUT,
+        write_timeout=MASTER_TIMEOUT,
+        connection_timeout=MASTER_TIMEOUT,
     )
 except RuntimeError:
     pass  # ignore if some EPICS object already created
 
 
-class MyFixedCam(SimDetectorCam):
-    pool_max_buffers = None
-    offset = ADComponent(EpicsSignalWithRBV, "Offset")
+class MySimDetectorCam(CamMixin, SimDetectorCam):
+    """triggering configuration and AcquireBusy support"""
 
 
 class MySimDetector(SingleTrigger, DetectorBase):
     """ADSimDetector"""
 
-    cam = ADComponent(MyFixedCam, "cam1:")
+    cam = ADComponent(MySimDetectorCam, "cam1:")
     hdf1 = ADComponent(
         AD_EpicsFileNameHDF5Plugin,
         "HDF1:",
@@ -92,11 +86,13 @@ class MySimDetector(SingleTrigger, DetectorBase):
 def adsimdet():
     "EPICS ADSimDetector."
     adsimdet = MySimDetector(IOC, name="adsimdet")
+    adsimdet.stage_sigs["cam.wait_for_plugins"] = "Yes"
     adsimdet.wait_for_connection(timeout=15)
     for plugin_name in "hdf1 jpeg1 tiff1".split():
         adsimdet.read_attrs.append(plugin_name)
         plugin = getattr(adsimdet, plugin_name)
-        AD_prime_plugin2(plugin)
+        if not AD_plugin_primed(plugin):
+            AD_prime_plugin2(plugin)
 
         # these settings are the caller's responsibility
         plugin.file_name.put("")  # control this in tests
@@ -122,6 +118,68 @@ def fname():
     yield fname
 
 
+@pytest.mark.parametrize(
+    # fmt: off
+    "plugin_name, spec",
+    [
+        ["hdf1", "AD_HDF5"],
+        ["jpeg1", "AD_JPEG"],
+        ["tiff1", "AD_TIFF"],
+    ],
+    # fmt: on
+)
+def test_AD_EpicsFileNameMixin(plugin_name, spec, adsimdet):
+    assert adsimdet is not None
+    assert plugin_name in dir(adsimdet)
+
+    # basic plugin tests
+    mixin = getattr(adsimdet, plugin_name)
+    assert AD_EpicsFileNameMixin is not None
+    assert isinstance(mixin, AD_EpicsFileNameMixin)
+    assert "filestore_spec" in dir(mixin)
+    assert mixin.filestore_spec == spec
+    assert isinstance(mixin.stage_sigs, dict)
+    assert len(mixin.stage_sigs) >= 4
+    assert "capture" in mixin.stage_sigs
+
+    # configuration prescribed for the user
+    user_settings = dict(
+        array_counter=0,
+        auto_increment=1,  # Yes
+        auto_save=1,  # Yes
+        create_directory=-5,
+        file_name="flotsam",
+        file_number=1 + int(10 * random.random()),
+        file_path=f"{pathlib.Path(tempfile.mkdtemp())}/",  # ALWAYS ends with "/"
+        file_template=f"%s%s_%2.2d.{plugin_name}",
+        num_capture=1 + int(10 * random.random()),
+    )
+    if plugin_name == "hdf1":
+        user_settings["compression"] = "zlib"
+
+    # add the settings
+    for k, v in user_settings.items():
+        getattr(mixin, k).put(v)
+
+    adsimdet.stage()
+    if plugin_name == "hdf1":  # Why special case?
+        user_settings["file_number"] += 1  # the IOC will do the same
+    assert list(mixin.stage_sigs.keys())[-1] == "capture"
+    for k, v in user_settings.items():
+        assert getattr(mixin, k).get() == v, f"{plugin_name=} {k=}  {v=}"
+
+    assert mixin.get_frames_per_point() == user_settings["num_capture"]
+
+    filename, read_path, write_path = mixin.make_filename()
+    assert isinstance(filename, str)
+    assert isinstance(read_path, str)
+    assert isinstance(write_path, str)
+    assert filename == user_settings["file_name"]
+    assert read_path.startswith(datetime.datetime.now().strftime(READ_PATH_TEMPLATE))
+    assert write_path == user_settings["file_path"]
+    adsimdet.unstage()
+
+
 @pytest.mark.parametrize(
     "plugin_name, plugin_class",
     [
@@ -155,29 +213,72 @@ def test_stage(plugin_name, plugin_class, adsimdet, fname):
         ["tiff1", AD_EpicsFileNameTIFFPlugin],
     ],
 )
-def test_acquire(plugin_name, plugin_class, adsimdet, fname):
+def test_bp_count_custom_name(plugin_name, plugin_class, adsimdet, fname):
     assert isinstance(plugin_name, str)
 
     plugin = getattr(adsimdet, plugin_name)
     assert isinstance(plugin, plugin_class)
 
-    plugin.file_name.put(fname)
-    ext = dict(hdf1="h5", jpeg1="jpg", tiff1="tif")[plugin_name]
-    plugin.file_template.put(f"%s%s_%4.4d.{ext}")
-    time.sleep(0.2)
+    def prepare_count(file_path, file_name, n_images):
+        # fmt: off
+        yield from bps.mv(
+            plugin.array_counter, 0,
+            plugin.auto_increment, "Yes",
+            plugin.auto_save, "Yes",
+            plugin.create_directory, -5,
+            plugin.dropped_arrays, 0,
+            plugin.file_name, file_name,
+            plugin.file_path, file_path,
+            plugin.num_capture, n_images,
+            adsimdet.cam.num_images, n_images,
+            adsimdet.cam.acquire_time, 0.001,
+            adsimdet.cam.acquire_period, 0.002,
+            adsimdet.cam.image_mode, "Single",
+            # plugin.file_template  # pre-configured
+        )
+        # fmt: on
 
-    adsimdet.stage()
-    adsimdet.trigger()
-    time.sleep(0.005)
-    while adsimdet.cam.acquire.get(use_monitor=False) not in (0, "Done"):
-        time.sleep(0.005)
-    adsimdet.unstage()
+    RE = bluesky.RunEngine({})
+    NUM_FRAMES = 1
+    RE(prepare_count(f"{plugin.write_path_template}/", fname, NUM_FRAMES))
+    assert plugin.auto_increment.get() in (1, "Yes")
+    assert plugin.auto_save.get() in (1, "Yes")
+    assert plugin.create_directory.get() == -5
+    assert plugin.file_name.get() == fname
+    assert plugin.file_path.get() == f"{plugin.write_path_template}/"
+    assert plugin.file_template.get() not in ("", "/")
+    assert plugin.num_capture.get() == NUM_FRAMES
+    assert adsimdet.cam.num_images.get() == NUM_FRAMES
+    assert adsimdet.cam.acquire_time.get() == 0.001
+    assert adsimdet.cam.acquire_period.get() == 0.002
+    assert adsimdet.cam.image_mode.get() in (0, "Single")
 
+    # remember this for testing later
+    next_file_number = plugin.file_number.get()
+
+    # acquire the image (using standard technique)
+    uids = RE(bp.count([adsimdet]))
+    assert len(uids) == 1
+    assert plugin.file_number.get() > next_file_number
+    assert not adsimdet.cam.is_busy
+
+    assert plugin.dropped_arrays.get() == 0
+    assert plugin.array_counter.get() == NUM_FRAMES
+    assert plugin.num_captured.get() == NUM_FRAMES
+
+    # diagnostic per issue #696
+    ffname = plugin.full_file_name.get(use_monitor=False).strip()
+    if ffname in ("", "/"):
+        # second chance, race condition?  don't trust the CA monitor cache
+        ffname = plugin.full_file_name.get(use_monitor=False).strip()
+    assert ffname not in ("", "/"), str(ffname)
+
+    # local file-system image file name
     lfname = AD_full_file_name_local(plugin)
     assert lfname is not None
     assert lfname.exists(), lfname
     assert isinstance(lfname, pathlib.Path)
-    assert str(lfname).find(fname) > 0, f"{lfname=}  {fname=}"
+    assert fname in str(lfname), f"{lfname=}  {fname=}"
 
 
 def test_full_file_name_local(adsimdet, fname):
@@ -190,7 +291,7 @@ def test_full_file_name_local(adsimdet, fname):
     adsimdet.unstage()
 
     lfname = AD_full_file_name_local(plugin)
-    assert lfname is not None
+    assert lfname is not None, str(plugin)
     assert lfname.exists(), lfname
     assert isinstance(lfname, pathlib.Path)
     assert str(lfname).find(fname) > 0
@@ -208,7 +309,7 @@ def test_no_file_path(adsimdet, fname):
 
     with pytest.raises(OSError) as exc:
         adsimdet.stage()
-    assert str(exc.value).endswith("'' does not exist on IOC.")
+    assert str(exc.value).endswith(" does not exist on IOC.")
     adsimdet.unstage()
 
 
@@ -228,21 +329,8 @@ def test_file_numbering(adsimdet, fname):
     adsimdet.unstage()
     full_file_name = plugin.full_file_name.get()
     assert isinstance(full_file_name, str)
-    assert full_file_name.find(fname) > 0
-    assert full_file_name.endswith(
-        f"{fname}_{file_number:04d}.h5"
-    ), f"{full_file_name=} {fname}_{file_number:04d}.h5"
-
-    # prepare for file name with no file number
-    # stage it: show that does not work
-    file_number = plugin.file_number.get()
-    plugin.stage_sigs["file_template"] = "%s%s.h5"
-    adsimdet.stage()
-    adsimdet.unstage()
-    full_file_name = plugin.full_file_name.get()
-    assert isinstance(full_file_name, str)
-    assert full_file_name.find(fname) > 0
-    # still not changed
+    assert len(full_file_name) > 0
+    assert full_file_name.find(fname) > 0, str(full_file_name)
     assert full_file_name.endswith(
         f"{fname}_{file_number:04d}.h5"
     ), f"{full_file_name=} {fname}_{file_number:04d}.h5"
