@@ -7,13 +7,12 @@ sscan Record plans
    ~sscan_1D
 """
 
-from collections import OrderedDict
 import time
+from collections import OrderedDict
 
 from bluesky import plan_stubs as bps
 from ophyd import DeviceStatus
 
-
 new_data = False  # sscan has new data, boolean
 inactive_deadline = 0  # sscan timeout, absolute time.time()
 
@@ -152,7 +151,7 @@ def sscan_1D(
     _md = dict(plan_name="sscan_1D")
     _md.update(md or {})
 
-    yield from bps.open_run(_md)  # start data collection
+    uid = yield from bps.open_run(_md)  # start data collection
     yield from bps.mv(sscan.execute_scan, 1)  # start sscan
     started = True
 
@@ -195,14 +194,14 @@ def sscan_1D(
     elapsed = time.time() - t0
     print(f"total time for sscan_1D: {elapsed} s")
 
-    return sscan_status
+    return uid
 
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 # -----------------------------------------------------------------------------
