@@ -10,8 +10,9 @@ Public Structures
 
 """
 
-from ophyd.device import Device, Component
 from ophyd import EpicsSignal
+from ophyd.device import Component
+from ophyd.device import Device
 
 
 class BusyRecord(Device):
@@ -28,7 +29,7 @@ class BusyRecord(Device):
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
 # Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
