@@ -16,6 +16,7 @@ EXAMPLES::
 
 .. autosummary::
 
+    ~UserCalcN
     ~UserCalcsDevice
     ~SwaitRecord
     ~SwaitRecordChannel
@@ -27,28 +28,19 @@ EXAMPLES::
 :see: https://htmlpreview.github.io/?https://raw.githubusercontent.com/epics-modules/calc/R3-6-1/documentation/swaitRecord.html
 """
 
-# -----------------------------------------------------------------------------
-# :author:    Pete R. Jemian
-# :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
-#
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
-#
-# The full license is in the file LICENSE.txt, distributed with this software.
-# -----------------------------------------------------------------------------
-
 
 from collections import OrderedDict
-from ophyd import Device
+
 from ophyd import Component as Cpt
+from ophyd import Device
 from ophyd import DynamicDeviceComponent as DDC
-from ophyd import FormattedComponent as FC
 from ophyd import EpicsSignal
+from ophyd import FormattedComponent as FC
 from ophyd.signal import EpicsSignalBase
 
-from ._common import EpicsRecordDeviceCommonAll
 from .. import utils as APS_utils
-
+from ._common import EpicsRecordDeviceCommonAll
+from ._common import EpicsSynAppsRecordEnableMixin
 
 CHANNEL_LETTERS_LIST = "A B C D E F G H I J K L".split()
 
@@ -60,7 +52,7 @@ class SwaitRecordChannel(Device):
     .. index:: Ophyd Device; synApps SwaitRecordChannel
     """
 
-    input_value = FC(EpicsSignal, "{self.prefix}.{self._ch_letter}", kind="normal")
+    input_value = FC(EpicsSignal, "{self.prefix}.{self._ch_letter}", kind="config")
     input_pv = FC(EpicsSignal, "{self.prefix}.IN{self._ch_letter}N", kind="config")
     input_trigger = FC(EpicsSignal, "{self.prefix}.IN{self._ch_letter}P", kind="config")
     read_attrs = [
@@ -97,8 +89,6 @@ class SwaitRecord(EpicsRecordDeviceCommonAll):
         ~reset
 
     """
-    enable = Cpt(EpicsSignal, "Enable", kind="config")
-
     precision = Cpt(EpicsSignal, ".PREC", kind="config")
     high_operating_range = Cpt(EpicsSignal, ".HOPR", kind="config")
     low_operating_range = Cpt(EpicsSignal, ".LOPR", kind="config")
@@ -149,6 +139,10 @@ class SwaitRecord(EpicsRecordDeviceCommonAll):
         self.read_attrs.append("calculated_value")
 
 
+class UserCalcN(EpicsSynAppsRecordEnableMixin, SwaitRecord):
+    """Single instance of the userCalcN database."""
+
+
 class UserCalcsDevice(Device):
     """
     EPICS synApps XXX IOC setup of userCalcs: ``$(P):userCalc$(N)``
@@ -162,16 +156,16 @@ class UserCalcsDevice(Device):
     """
 
     enable = Cpt(EpicsSignal, "userCalcEnable", kind="config")
-    calc1 = Cpt(SwaitRecord, "userCalc1")
-    calc2 = Cpt(SwaitRecord, "userCalc2")
-    calc3 = Cpt(SwaitRecord, "userCalc3")
-    calc4 = Cpt(SwaitRecord, "userCalc4")
-    calc5 = Cpt(SwaitRecord, "userCalc5")
-    calc6 = Cpt(SwaitRecord, "userCalc6")
-    calc7 = Cpt(SwaitRecord, "userCalc7")
-    calc8 = Cpt(SwaitRecord, "userCalc8")
-    calc9 = Cpt(SwaitRecord, "userCalc9")
-    calc10 = Cpt(SwaitRecord, "userCalc10")
+    calc1 = Cpt(UserCalcN, "userCalc1")
+    calc2 = Cpt(UserCalcN, "userCalc2")
+    calc3 = Cpt(UserCalcN, "userCalc3")
+    calc4 = Cpt(UserCalcN, "userCalc4")
+    calc5 = Cpt(UserCalcN, "userCalc5")
+    calc6 = Cpt(UserCalcN, "userCalc6")
+    calc7 = Cpt(UserCalcN, "userCalc7")
+    calc8 = Cpt(UserCalcN, "userCalc8")
+    calc9 = Cpt(UserCalcN, "userCalc9")
+    calc10 = Cpt(UserCalcN, "userCalc10")
 
     def reset(self):  # lgtm [py/similar-function]
         """set all fields to default values"""
@@ -404,3 +398,13 @@ def setup_incrementer_swait(swait, scan=None, limit=100000):
         "calculated_value",
     ]
     swait.hints = {"fields": swait.read_attrs}
+
+# -----------------------------------------------------------------------------
+# :author:    Pete R. Jemian
+# :email:     jemian@anl.gov
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
+#
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+# -----------------------------------------------------------------------------
