@@ -1,12 +1,12 @@
-from ophyd import EpicsSignal
 import pytest
+from ophyd import EpicsSignal
 
-from ..sseq import SseqRecord
-from ..sseq import sseqRecordStep
-from ..sseq import UserStringSequenceDevice
-from ...tests import common_attribute_quantities_test
 from ...tests import IOC
-from ...tests import short_delay_for_EPICS_IOC_database_processing
+from ...tests import common_attribute_quantities_test
+from ...tests import timed_pause
+from ..sseq import SseqRecord
+from ..sseq import UserStringSequenceDevice
+from ..sseq import sseqRecordStep
 
 
 @pytest.mark.parametrize(
@@ -42,7 +42,7 @@ def test_sseq_reset():
     assert isinstance(step, sseqRecordStep)
 
     step.reset()
-    short_delay_for_EPICS_IOC_database_processing()
+    timed_pause()
 
     assert step.input_pv.get() == ""
 
@@ -55,16 +55,6 @@ def test_sseq_reset():
     assert step.string_value.get() <= uptime.get()
 
     user.reset()
-    short_delay_for_EPICS_IOC_database_processing()
+    timed_pause()
     assert step.input_pv.get() == ""
     assert step.string_value.get() == f"{0:.5f}"
-
-# -----------------------------------------------------------------------------
-# :author:    Pete R. Jemian
-# :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
-#
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
-#
-# The full license is in the file LICENSE.txt, distributed with this software.
-# -----------------------------------------------------------------------------
