@@ -1,12 +1,12 @@
 import pytest
 
+from ...tests import IOC
+from ...tests import common_attribute_quantities_test
+from ...tests import timed_pause
 from ..calcout import CalcoutRecord
 from ..sub import SubRecord
-from ..sub import UserAverageN
 from ..sub import UserAverageDevice
-from ...tests import common_attribute_quantities_test
-from ...tests import IOC
-from ...tests import short_delay_for_EPICS_IOC_database_processing
+from ..sub import UserAverageN
 
 
 @pytest.fixture(scope="function")
@@ -79,7 +79,7 @@ def test_useraverage_reset(ave):
     ave.algorithm.put("FIT-LINE")
 
     ave.reset()
-    short_delay_for_EPICS_IOC_database_processing()
+    timed_pause()
     assert ave.initroutine.get() == "initSubAve"
     assert ave.subroutine.get() == "SubAve"
 
@@ -115,13 +115,3 @@ def test_useraverage_reset(ave):
 #     assert ave.busy.get(as_string=False) == 0
 #     # test that average = 0.5 as n -> inf
 #     assert abs(ave.averaged_value.get() - 0.5) <= 0.1
-
-# -----------------------------------------------------------------------------
-# :author:    Pete R. Jemian
-# :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
-#
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
-#
-# The full license is in the file LICENSE.txt, distributed with this software.
-# -----------------------------------------------------------------------------
