@@ -6,31 +6,23 @@ Public Structures
 
 .. autosummary::
 
+    ~UserTransformN
     ~UserTransformsDevice
     ~TransformRecord
 """
 
-# -----------------------------------------------------------------------------
-# :author:    Pete R. Jemian
-# :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
-#
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
-#
-# The full license is in the file LICENSE.txt, distributed with this software.
-# -----------------------------------------------------------------------------
-
 from collections import OrderedDict
-from ophyd import Device
+
 from ophyd import Component as Cpt
+from ophyd import Device
 from ophyd import DynamicDeviceComponent as DDC
-from ophyd import FormattedComponent as FC
 from ophyd import EpicsSignal
 from ophyd import EpicsSignalRO
+from ophyd import FormattedComponent as FC
 
-from ._common import EpicsRecordDeviceCommonAll
 from .. import utils as APS_utils
-
+from ._common import EpicsRecordDeviceCommonAll
+from ._common import EpicsSynAppsRecordEnableMixin
 
 CHANNEL_LETTERS_LIST = "A B C D E F G H I J K L M N O P".split()
 
@@ -126,6 +118,10 @@ class TransformRecord(EpicsRecordDeviceCommonAll):
         self.read_attrs = ["channels.%s" % c for c in CHANNEL_LETTERS_LIST]
 
 
+class UserTransformN(EpicsSynAppsRecordEnableMixin, TransformRecord):
+    """Single instance of the userTranN database."""
+
+
 class UserTransformsDevice(Device):
     """
     EPICS synApps XXX IOC setup of userTransforms: ``$(P):userTran$(N)``
@@ -134,16 +130,16 @@ class UserTransformsDevice(Device):
     """
 
     enable = Cpt(EpicsSignal, "userTranEnable", kind="config")
-    transform1 = Cpt(TransformRecord, "userTran1")
-    transform2 = Cpt(TransformRecord, "userTran2")
-    transform3 = Cpt(TransformRecord, "userTran3")
-    transform4 = Cpt(TransformRecord, "userTran4")
-    transform5 = Cpt(TransformRecord, "userTran5")
-    transform6 = Cpt(TransformRecord, "userTran6")
-    transform7 = Cpt(TransformRecord, "userTran7")
-    transform8 = Cpt(TransformRecord, "userTran8")
-    transform9 = Cpt(TransformRecord, "userTran9")
-    transform10 = Cpt(TransformRecord, "userTran10")
+    transform1 = Cpt(UserTransformN, "userTran1")
+    transform2 = Cpt(UserTransformN, "userTran2")
+    transform3 = Cpt(UserTransformN, "userTran3")
+    transform4 = Cpt(UserTransformN, "userTran4")
+    transform5 = Cpt(UserTransformN, "userTran5")
+    transform6 = Cpt(UserTransformN, "userTran6")
+    transform7 = Cpt(UserTransformN, "userTran7")
+    transform8 = Cpt(UserTransformN, "userTran8")
+    transform9 = Cpt(UserTransformN, "userTran9")
+    transform10 = Cpt(UserTransformN, "userTran10")
 
     def reset(self):  # lgtm [py/similar-function]
         """set all fields to default values"""
@@ -159,3 +155,13 @@ class UserTransformsDevice(Device):
         self.transform10.reset()
         self.read_attrs = ["transform%d" % (c + 1) for c in range(10)]
         self.read_attrs.insert(0, "enable")
+
+# -----------------------------------------------------------------------------
+# :author:    Pete R. Jemian
+# :email:     jemian@anl.gov
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
+#
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+# -----------------------------------------------------------------------------
