@@ -8,16 +8,15 @@ Device information
 """
 
 
+import datetime
+import logging
 from collections import defaultdict
+
+import pandas as pd
 from ophyd import Device
 from ophyd import Signal
 from ophyd.signal import EpicsSignalBase
 
-import datetime
-import logging
-import pandas as pd
-
-
 logger = logging.getLogger(__name__)
 pd.set_option("display.max_rows", None)
 
@@ -143,9 +142,12 @@ def listdevice(
         reading = obj.read()
         signals = [s for s in signals if s.name in reading]
     else:
+        # fmt: off
         raise KeyError(
-            f"Unknown scope='{scope}'." " Must be one of None, 'full', 'epics', 'read'"
+            f"Unknown scope='{scope}'."
+            " Must be one of None, 'full', 'epics', 'read'"
         )
+        # fmt: on
 
     # in EPICS, an uninitialized PV has a timestamp of 1990-01-01 UTC
     UNINITIALIZED = datetime.datetime.timestamp(
@@ -161,7 +163,11 @@ def listdevice(
             ts = getattr(signal, "timestamp", 0)
             if show_ancient or (ts >= UNINITIALIZED):
                 if cname:
-                    dd["name"].append(f"{obj.name}.{signal.dotted_name}")
+                    head = obj
+                    while head.dotted_name != "":
+                        # walk back to the head
+                        head = head.parent
+                    dd["name"].append(f"{head.name}.{signal.dotted_name}")
                 if dname:
                     dd["data name"].append(signal.name)
                 if show_pv:
@@ -175,9 +181,9 @@ def listdevice(
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
-# Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 # -----------------------------------------------------------------------------
