@@ -12,26 +12,24 @@ Directory of bluesky runs
    ~summarize_runs
 """
 
+import dataclasses
+import datetime
+import logging
+import time
+import typing
+from collections import defaultdict
+
 import databroker
 import databroker._drivers.mongo_normalized
 import databroker._drivers.msgpack
 import databroker.queries
-import dataclasses
-import datetime
-import logging
 import pandas as pd
 import pyRestTable
-import time
-import typing
-
-from collections import defaultdict
-
-from .query import db_query
 
 from ._core import FIRST_DATA
 from ._core import LAST_DATA
 from ._core import MONGO_CATALOG_CLASSES
-
+from .query import db_query
 
 logger = logging.getLogger(__name__)
 
@@ -94,9 +92,7 @@ def getRunData(scan_id, db=None, stream="primary", query=None, use_v1=True):
     raise AttributeError(f"No such stream '{stream}' in run '{scan_id}'.")
 
 
-def getRunDataValue(
-    scan_id, key, db=None, stream="primary", query=None, idx=-1, use_v1=True
-):
+def getRunDataValue(scan_id, key, db=None, stream="primary", query=None, idx=-1, use_v1=True):
     """
     Convenience function to get value of key in run stream.
 
@@ -156,9 +152,7 @@ def getRunDataValue(
         _idx = str(idx).lower()
 
     if isinstance(_idx, str) and _idx not in "all mean".split():
-        raise KeyError(
-            f"Did not understand 'idx={idx}', use integer, 'all', or 'mean'."
-        )
+        raise KeyError(f"Did not understand 'idx={idx}', use integer, 'all', or 'mean'.")
 
     stream = stream or "primary"
 
@@ -174,9 +168,7 @@ def getRunDataValue(
         return data.mean()
     elif (0 <= _idx < len(data)) or (_idx < 0):
         return data.values[_idx]
-    raise KeyError(
-        f"Cannot reference idx={idx} in scan {scan_id} stream'{stream}' key={key}."
-    )
+    raise KeyError(f"Cannot reference idx={idx} in scan {scan_id} stream'{stream}' key={key}.")
 
 
 def listRunKeys(
@@ -284,16 +276,17 @@ class ListRuns:
     """
 
     cat: object = None
-    query: dict = None
-    keys: str = None
+    query: object = None
+    keys: object = None
     missing: str = ""
     num: int = 20
     reverse: bool = True
-    since: str = None
+    since: object = None
     sortby: str = "time"
     timefmt: str = "%Y-%m-%d %H:%M:%S"
-    until: str = None
+    until: object = None
     ids: "typing.Any" = None
+    hints_override: bool = False
 
     _default_keys = "scan_id time plan_name detectors"
 
@@ -321,7 +314,7 @@ class ListRuns:
         It is not possible to ``sortby`` the dotted-key syntax
         at this time.
         """
-        v = None
+        v = self.missing
         if key == "time":
             v = md["start"][key]
             if self.timefmt != "raw":
@@ -339,7 +332,10 @@ class ListRuns:
                 if key == "time" and self.timefmt != "raw":
                     ts = datetime.datetime.fromtimestamp(v)
                     v = ts.strftime(self.timefmt)
-        return v or self.missing
+        hints = md["start"].get("hints", {})
+        if (v == self.missing or self.hints_override) and key in hints:
+            v = hints[key]
+        return v
 
     def _check_cat(self):
         from . import getCatalog
@@ -413,8 +409,7 @@ class ListRuns:
     def _check_keys(self):
         """Check that self.keys is a list of strings."""
         self.keys = self.keys or self._default_keys
-        if isinstance(self.keys, str) and self.keys.find(" ") >= 0:
-            # convert a space-delimited string of names
+        if isinstance(self.keys, str):
             self.keys = self.keys.split()
 
     def to_dataframe(self):
@@ -449,6 +444,7 @@ def listruns(
     timefmt="%Y-%m-%d %H:%M:%S",
     until=None,
     ids=None,
+    hints_override=False,
     **query,
 ):
     """
@@ -470,6 +466,10 @@ def listruns(
         *str*:
         Test to report when a value is not available.
         (default: ``""``)
+    hints_override *bool*:
+        For a key that appears in both the metadata and the hints,
+        override the metadata value if the same key is found in the hints.
+        (default: ``False``)
     ids
         *[int]* or *[str]*:
         List of ``uid`` or ``scan_id`` value(s).
@@ -558,12 +558,14 @@ def listruns(
         keys=keys,
         missing=missing,
         num=num,
+        query=query,
         reverse=reverse,
         since=since,
         sortby=sortby,
         timefmt=timefmt,
         until=until,
         ids=ids,
+        hints_override=hints_override,
     )
 
     tablefmt = tablefmt or "dataframe"
@@ -654,7 +656,7 @@ def summarize_runs(since=None, db=None):
 # -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2022, UChicago Argonne, LLC
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
 #
 # Distributed under the terms of the Argonne National Laboratory Open Source License.
 #
