@@ -12,7 +12,6 @@ Miscellaneous Support
    ~listobjects
    ~pairwise
    ~print_RE_md
-   ~print_snapshot_list
    ~redefine_motor_position
    ~replay
    ~run_in_thread
@@ -24,24 +23,24 @@ Miscellaneous Support
    ~unix
 """
 
-import databroker
 import logging
-import ophyd
-import pyRestTable
 import re
 import subprocess
 import sys
 import threading
 import time
-
-from bluesky import plan_stubs as bps
-from bluesky.callbacks.best_effort import BestEffortCallback
+import warnings
 from collections import OrderedDict
 
+import databroker
+import ophyd
+import pyRestTable
+from bluesky import plan_stubs as bps
+from bluesky.callbacks.best_effort import BestEffortCallback
+
+from ..callbacks import spec_file_writer
 from ._core import MAX_EPICS_STRINGOUT_LENGTH
 from .profile_support import ipython_shell_namespace
-from ..callbacks import spec_file_writer
-
 
 logger = logging.getLogger(__name__)
 
@@ -211,16 +210,17 @@ def replay(headers, callback=None, sort=True):
     PARAMETERS
 
     headers
-        *scan* or *[scan]* :
-        Scan(s) to be replayed through callback.
-        A *scan* is an instance of a Bluesky ``databroker.Header``.
+        *run* or *[run]* :
+        Run(s) to be replayed through callback. A *run* is an instance of a
+        Bluesky ``databroker.core.BlueskyRun` (or the older
+        ``databroker.Header``).
         see: https://nsls-ii.github.io/databroker/api.html?highlight=header#header-api
 
     callback
-        *scan* or *[scan]* :
-        The Bluesky callback to handle the stream of documents from a
-        scan. If ``None``, then use the `bec` (BestEffortCallback) from
-        the IPython shell.
+        *run* or *[run]* :
+        The Bluesky callback to handle the stream of documents from a run. If
+        ``None``, then use the `bec` (BestEffortCallback) from the IPython
+        shell.
         (default:``None``)
 
     sort
@@ -237,9 +237,18 @@ def replay(headers, callback=None, sort=True):
     )
     # fmt: on
     _headers = headers  # do not mutate the input arg
-    if isinstance(_headers, databroker.Header):
+    if not isinstance(_headers, (list, tuple)):
         _headers = [_headers]
 
+    def as_header(run):
+        if hasattr(run, "start"):
+            return run  # this is a cat.v1 header
+        # convert cat.v2 run to cat.v1 header
+        uid = run.metadata["start"]["uid"]
+        return run.cat.v1[uid]
+
+    runs = [as_header(h) for h in _headers]
+
     def increasing_time_sorter(run):
         return run.start["time"]
 
@@ -254,7 +263,7 @@ def replay(headers, callback=None, sort=True):
     }[sort]
     # fmt: on
 
-    for h in sorted(_headers, key=sorter):
+    for h in sorted(runs, key=sorter):
         if not isinstance(h, databroker.Header):
             # fmt: off
             raise TypeError(
@@ -569,98 +578,18 @@ def connect_pvlist(pvlist, wait=True, timeout=2, poll_interval=0.1):
     return obj_dict
 
 
-def print_snapshot_list(db, printing=True, **search_criteria):
-    """
-    print (stdout) a list of all snapshots in the databroker
-
-    USAGE::
-
-        print_snapshot_list(db, )
-        print_snapshot_list(db, purpose="this is an example")
-        print_snapshot_list(db, since="2018-12-21", until="2019")
-
-    EXAMPLE::
-
-        In [16]: from apstools.utils import print_snapshot_list
-            ...: from apstools.callbacks import SnapshotReport
-            ...: print_snapshot_list(db, since="2018-12-21", until="2019")
-            ...:
-        = ======== ========================== ==================
-        # uid      date/time                  purpose
-        = ======== ========================== ==================
-        0 d7831dae 2018-12-21 11:39:52.956904 this is an example
-        1 5049029d 2018-12-21 11:39:30.062463 this is an example
-        2 588e0149 2018-12-21 11:38:43.153055 this is an example
-        = ======== ========================== ==================
-
-        In [17]: SnapshotReport().print_report(db["5049029d"])
-
-        ========================================
-        snapshot: 2018-12-21 11:39:30.062463
-        ========================================
-
-        example: example 2
-        hints: {}
-        iso8601: 2018-12-21 11:39:30.062463
-        look: can snapshot text and arrays too
-        note: no commas in metadata
-        plan_description: archive snapshot of ophyd Signals (usually EPICS PVs)
-        plan_name: snapshot
-        plan_type: generator
-        purpose: this is an example
-        scan_id: 1
-        software_versions: {
-            'python':
-                '''3.6.2 |Continuum Analytics, Inc.| (default, Jul 20 2017, 13:51:32)
-                [GCC 4.4.7 20120313 (Red Hat 4.4.7-1)]''',
-            'PyEpics': '3.3.1',
-            'bluesky': '1.4.1',
-            'ophyd': '1.3.0',
-            'databroker': '0.11.3',
-            'apstools': '0.0.38'
-            }
-        time: 1545413970.063167
-        uid: 5049029d-075c-453c-96d2-55431273852b
-
-        ========================== ====== ================ ===================
-        timestamp                  source name             value
-        ========================== ====== ================ ===================
-        2018-12-20 18:24:34.220028 PV     compress         [0.1, 0.2, 0.3]
-        2018-12-13 14:49:53.121188 PV     gov:HOSTNAME     otz.aps.anl.gov
-        2018-12-21 11:39:24.268148 PV     gov:IOC_CPU_LOAD 0.22522317161410768
-        2018-12-21 11:39:24.268151 PV     gov:SYS_CPU_LOAD 9.109026666525944
-        2018-12-21 11:39:30.017643 PV     gov:iso8601      2018-12-21T11:39:30
-        2018-12-13 14:49:53.135016 PV     otz:HOSTNAME     otz.aps.anl.gov
-        2018-12-21 11:39:27.705304 PV     otz:IOC_CPU_LOAD 0.1251210270549924
-        2018-12-21 11:39:27.705301 PV     otz:SYS_CPU_LOAD 11.611234438304471
-        2018-12-21 11:39:30.030321 PV     otz:iso8601      2018-12-21T11:39:30
-        ========================== ====== ================ ===================
-
-        exit_status: success
-        num_events: {'primary': 1}
-        run_start: 5049029d-075c-453c-96d2-55431273852b
-        time: 1545413970.102147
-        uid: 6c1b2100-1ef6-404d-943e-405da9ada882
-
-    """
-    t = pyRestTable.Table()
-    t.addLabel("#")
-    t.addLabel("uid")
-    t.addLabel("date/time")
-    t.addLabel("#items")
-    t.addLabel("purpose")
-    search_criteria["plan_name"] = "snapshot"
-    for i, h in enumerate(db(**search_criteria)):
-        uid = h.start["uid"].split("-")[0]
-        n = len(list(h.start.keys()))
-        t.addRow((i, uid, h.start["iso8601"], n, h.start["purpose"]))
-    if printing:
-        print(t)
-    return t
-
-
 def redefine_motor_position(motor, new_position):
     """Set EPICS motor record's user coordinate to ``new_position``."""
     yield from bps.mv(motor.set_use_switch, 1)
     yield from bps.mv(motor.user_setpoint, new_position)
     yield from bps.mv(motor.set_use_switch, 0)
+
+# -----------------------------------------------------------------------------
+# :author:    Pete R. Jemian
+# :email:     jemian@anl.gov
+# :copyright: (c) 2017-2023, UChicago Argonne, LLC
+#
+# Distributed under the terms of the Argonne National Laboratory Open Source License.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+# -----------------------------------------------------------------------------
