@@ -2,23 +2,26 @@
 Unit tests for :mod:`~apstools._utils.device_info`.
 """
 
+import pandas as pd
+import pytest
 from ophyd import Component
 from ophyd import Device
 from ophyd import EpicsMotor
 from ophyd import Signal
 from ophyd.signal import EpicsSignalBase
-import pandas as pd
-import pytest
 
 from ...devices import SwaitRecord
+from ...tests import MASTER_TIMEOUT
 from ..device_info import _list_epics_signals
 from ..device_info import listdevice
 
-
 # set default timeout for all EpicsSignal connections & communications
 try:
     EpicsSignalBase.set_defaults(
-        auto_monitor=True, timeout=60, write_timeout=60, connection_timeout=60,
+        auto_monitor=True,
+        timeout=MASTER_TIMEOUT,
+        write_timeout=MASTER_TIMEOUT,
+        connection_timeout=MASTER_TIMEOUT,
     )
 except RuntimeError:
     pass  # ignore if some EPICS object already created
@@ -103,8 +106,8 @@ def test__list_epics_signals(obj, length, ref):
         (None, 0, "value", 0.0),
         ("epics", 0, "data name", "motor"),
         ("epics", 0, "value", 0.0),
-        ("epics", 6, "data name", "motor_velocity"),
-        ("epics", 6, "value", 1.0),
+        ("epics", 7, "data name", "motor_acceleration"),
+        ("epics", 7, "value", 0.2),
         ("epics", 8, "data name", "motor_motor_egu"),
         ("epics", 8, "value", "degrees"),
         ("full", 0, "data name", "motor"),
@@ -117,6 +120,8 @@ def test__list_epics_signals(obj, length, ref):
 )  # scope: full epics read
 def test_spotchecks(scope, row, column, value):
     assert calcs.connected
+    if round(motor.position, 2) != 0:
+        motor.move(0)
     result = listdevice(motor, scope=scope)
     assert isinstance(result, pd.DataFrame)
     assert column in result
@@ -140,3 +145,26 @@ def test_spotchecks(scope, row, column, value):
 def test_listdevice_filters(device, scope, ancient, length):
     result = listdevice(device, scope, show_ancient=ancient)
     assert len(result) == length
+
+
+@pytest.mark.parametrize(
+    "device, scope, cnames",
+    [
+        (calcs, "full", [
+            "calcs.signals.allowed",
+            "calcs.signals.background",
+            "calcs.signals.tolerance",
+            "calcs.signals.visible",
+        ]),
+        # This is the case that produced issue 640
+        (calcs.signals, "full", [
+            "calcs.signals.allowed",
+            "calcs.signals.background",
+            "calcs.signals.tolerance",
+            "calcs.signals.visible",
+        ]),
+    ],
+)
+def test_listdevice_cname(device, scope, cnames):
+    result = listdevice(device, scope, show_ancient=False, cname=True)
+    assert result["name"].tolist() == cnames
