@@ -2,11 +2,12 @@
 test override_parameters module
 """
 
-from apstools.utils import OverrideParameters
 import numpy as np
 import pandas as pd
 import pytest
 
+from ...utils import OverrideParameters
+
 
 @pytest.fixture(scope="function")
 def overrides():
